////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// File:        dvr_simple.h
// Function:    Sensoray DVR communication protocol, simplified for embedded use
// Copyright:   Copyright (c) Sensoray Co., Inc 2020
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef _INC_DVR_SIMPLE_H_
#define _INC_DVR_SIMPLE_H_

#include "dvr_protocol.h"

typedef unsigned char   u8;     // 8-bit unsigned
typedef unsigned short  u16;    // 16-bit unsigned
typedef   signed short  s16;    // 16-bit signed
typedef unsigned int    u32;    // 32-bit unsigned


extern int open_com_port(void);
extern void close_com_port(void);

// instead of returning status with every function call, use this function as needed
u32 dvr_get_last_status(void);

// General
int dvr_get_api_version(u32 *version);
int dvr_get_hardware_info(u32 *hw_version, char *model);
int dvr_get_dvr_status(u32 *status);
int dvr_enable_event_reporting(u8 clientid, u32 eventflags, u16 port);
int dvr_set_display_mode(DVR_DISPLAY_MODE mode);
int dvr_get_general_settings(DVR_DISPLAY_MODE *mode);

// Date/time
int dvr_set_datetime(u32 datetime);
int dvr_get_datetime(u32 *datetime);

// Video Record
int dvr_record_start(DVR_STORAGE_DEVID devflags, DVR_VIDREC_FORMAT format,
                     u16 bitrate, char *filename);
int dvr_record_stop(void);
int dvr_record_pause(void);
int dvr_record_resume(void);

// Video Play
int dvr_play_skip(s16 seconds);
int dvr_play_start(DVR_STORAGE_DEVID dev, const char *filename);
int dvr_play_stop(void);
int dvr_play_pause(void);
int dvr_play_resume(void);
int dvr_set_play_speed(u8 speed);
int dvr_get_play_settings(u8 *speed);

// Snapshot
int dvr_view_snapshot(DVR_STORAGE_DEVID dev, const char *filename);
int dvr_grab_snapshot(DVR_STORAGE_DEVID dev, u8 quality, char *filename);
int dvr_snapshot_stop(void);

// File system
int dvr_get_filecount(DVR_STORAGE_DEVID dev, u32 *count);
int dvr_get_filename(DVR_STORAGE_DEVID dev, u32 index, char *filename);
int dvr_delete_file(DVR_STORAGE_DEVID dev, const char *filename);
int dvr_rename_file(DVR_STORAGE_DEVID dev, const char *oldname, const char *newname);

// Video overlay
int dvr_set_osd_text(u8 region, char *text);
int dvr_set_osd_position(u8 region, u16 x, u16 y);
int dvr_set_osd_enable(u8 region, u8 enable);
int dvr_set_osd_background(u8 region, DVR_OSD_BGMODE bgmode);
int dvr_set_osd_clock_format(DVR_DATE_FORMAT datefmt, DVR_TIME_FORMAT timefmt);
int dvr_get_osd_clock_format(DVR_DATE_FORMAT *datefmt, DVR_TIME_FORMAT *timefmt);
int dvr_get_osd_settings(u8 region, u8 *enable, DVR_OSD_BGMODE *bgmode);
int dvr_osd_image_create(u8 region, u16 len);
int dvr_osd_image_upload_chunk(u8 region, u16 addr, u16 len, u8 *payload);
int dvr_osd_image_install(u8 region, DVR_OSD_IMAGETYPE imagetype);

// Video adjust
int dvr_set_video_adjust(DVR_VIDEO_ADJ_ID param_id, u8 value);
int dvr_get_video_adjust(DVR_VIDEO_ADJ_ID param_id, u8 *value);

// Video configuration
int dvr_set_video_config(u8 format, u8 deint, u8 decimate);
int dvr_get_video_config(u8 *format, u8 *deint, u8 *decimate);

// Audio
int dvr_set_audio_input(DVR_AUDIO_INPUT src);
int dvr_set_volume(DVR_AUDIO_CHAN chansel, u8 volume);
int dvr_set_mute(DVR_AUDIO_CHAN chansel, u8 mute);
int dvr_set_autovol(DVR_AUDIO_CHAN chansel, u8 agc);
int dvr_get_audio_settings(DVR_AUDIO_INPUT *src, u8 *Lvol, u8 *Rvol, u8 *Lmute, 
		u8 *Rmute, u8 *Lagc, u8 *Ragc);

// Incremental encoder
int dvr_set_encoder_counts(u8 chan, u32 count);
int dvr_get_encoder_counts(u8 chan, u32 *count);
int dvr_set_encoder_scalar(u8 chan, double scale);
int dvr_use_offboard_encoder(u8 chan, u8 enable);
int dvr_get_encoder_settings(u8 chan, double *scale, u8 *use_offboard);

// Menu system
int dvr_menu_display(u8 enable);
int dvr_menu_move_cursor(u8 up);
int dvr_menu_enter(u8 *menuid, u8 *executable, char *menutext);
int dvr_menu_exit(void);




#endif  // #ifndef _INC_DVR_SIMPLE_H_

